@@ -24,15 +24,14 @@ import (
 	"strings"
 	"text/tabwriter"
 
+	"github.com/containerd/containerd/v2/cmd/ctr/commands"
+	"github.com/containerd/errdefs"
 	"github.com/containerd/log"
-	"github.com/urfave/cli"
-
-	"github.com/containerd/containerd/cmd/ctr/commands"
-	"github.com/containerd/containerd/errdefs"
+	"github.com/urfave/cli/v2"
 )
 
 // Command is the cli command for managing namespaces
-var Command = cli.Command{
+var Command = &cli.Command{
 	Name:    "namespaces",
 	Aliases: []string{"namespace", "ns"},
 	Usage:   "Manage namespaces",
@@ -44,18 +43,18 @@ var Command = cli.Command{
 	},
 }
 
-var createCommand = cli.Command{
+var createCommand = &cli.Command{
 	Name:        "create",
 	Aliases:     []string{"c"},
 	Usage:       "Create a new namespace",
 	ArgsUsage:   "<name> [<key>=<value>]",
 	Description: "create a new namespace. it must be unique",
-	Action: func(context *cli.Context) error {
-		namespace, labels := commands.ObjectWithLabelArgs(context)
+	Action: func(cliContext *cli.Context) error {
+		namespace, labels := commands.ObjectWithLabelArgs(cliContext)
 		if namespace == "" {
 			return errors.New("please specify a namespace")
 		}
-		client, ctx, cancel, err := commands.NewClient(context)
+		client, ctx, cancel, err := commands.NewClient(cliContext)
 		if err != nil {
 			return err
 		}
@@ -65,17 +64,17 @@ var createCommand = cli.Command{
 	},
 }
 
-var setLabelsCommand = cli.Command{
+var setLabelsCommand = &cli.Command{
 	Name:        "label",
 	Usage:       "Set and clear labels for a namespace",
 	ArgsUsage:   "<name> [<key>=<value>, ...]",
 	Description: "set and clear labels for a namespace. empty value clears the label",
-	Action: func(context *cli.Context) error {
-		namespace, labels := commands.ObjectWithLabelArgs(context)
+	Action: func(cliContext *cli.Context) error {
+		namespace, labels := commands.ObjectWithLabelArgs(cliContext)
 		if namespace == "" {
 			return errors.New("please specify a namespace")
 		}
-		client, ctx, cancel, err := commands.NewClient(context)
+		client, ctx, cancel, err := commands.NewClient(cliContext)
 		if err != nil {
 			return err
 		}
@@ -90,21 +89,22 @@ var setLabelsCommand = cli.Command{
 	},
 }
 
-var listCommand = cli.Command{
+var listCommand = &cli.Command{
 	Name:        "list",
 	Aliases:     []string{"ls"},
 	Usage:       "List namespaces",
 	ArgsUsage:   "[flags]",
 	Description: "list namespaces",
 	Flags: []cli.Flag{
-		cli.BoolFlag{
-			Name:  "quiet, q",
-			Usage: "Print only the namespace name",
+		&cli.BoolFlag{
+			Name:    "quiet",
+			Aliases: []string{"q"},
+			Usage:   "Print only the namespace name",
 		},
 	},
-	Action: func(context *cli.Context) error {
-		quiet := context.Bool("quiet")
-		client, ctx, cancel, err := commands.NewClient(context)
+	Action: func(cliContext *cli.Context) error {
+		quiet := cliContext.Bool("quiet")
+		client, ctx, cancel, err := commands.NewClient(cliContext)
 		if err != nil {
 			return err
 		}
@@ -142,29 +142,30 @@ var listCommand = cli.Command{
 	},
 }
 
-var removeCommand = cli.Command{
+var removeCommand = &cli.Command{
 	Name:        "remove",
 	Aliases:     []string{"rm"},
 	Usage:       "Remove one or more namespaces",
 	ArgsUsage:   "<name> [<name>, ...]",
 	Description: "remove one or more namespaces. for now, the namespace must be empty",
 	Flags: []cli.Flag{
-		cli.BoolFlag{
-			Name:  "cgroup,c",
-			Usage: "Delete the namespace's cgroup",
+		&cli.BoolFlag{
+			Name:    "cgroup",
+			Aliases: []string{"c"},
+			Usage:   "Delete the namespace's cgroup",
 		},
 	},
-	Action: func(context *cli.Context) error {
+	Action: func(cliContext *cli.Context) error {
 		var exitErr error
-		client, ctx, cancel, err := commands.NewClient(context)
+		client, ctx, cancel, err := commands.NewClient(cliContext)
 		if err != nil {
 			return err
 		}
 		defer cancel()
 
-		opts := deleteOpts(context)
+		opts := deleteOpts(cliContext)
 		namespaces := client.NamespaceService()
-		for _, target := range context.Args() {
+		for _, target := range cliContext.Args().Slice() {
 			if err := namespaces.Delete(ctx, target, opts...); err != nil {
 				if !errdefs.IsNotFound(err) {
 					if exitErr == nil {
