@@ -28,16 +28,17 @@ import (
 	"os"
 	"path"
 	"strings"
+	"sync"
 
 	"github.com/containerd/errdefs"
 	"github.com/containerd/log"
 	"github.com/opencontainers/go-digest"
 	ocispec "github.com/opencontainers/image-spec/specs-go/v1"
+	"golang.org/x/sync/semaphore"
 
 	"github.com/containerd/containerd/v2/core/images"
 	"github.com/containerd/containerd/v2/core/remotes"
-	"github.com/containerd/containerd/v2/core/remotes/docker/schema1" //nolint:staticcheck // Ignore SA1019. Need to keep deprecated package for compatibility.
-	remoteerrors "github.com/containerd/containerd/v2/core/remotes/errors"
+	"github.com/containerd/containerd/v2/core/transfer"
 	"github.com/containerd/containerd/v2/pkg/reference"
 	"github.com/containerd/containerd/v2/pkg/tracing"
 	"github.com/containerd/containerd/v2/version"
@@ -141,6 +142,7 @@ type dockerResolver struct {
 	header        http.Header
 	resolveHeader http.Header
 	tracker       StatusTracker
+	config        transfer.ImageResolverOptions
 }
 
 // NewResolver returns a new resolver to a Docker registry
@@ -155,9 +157,6 @@ func NewResolver(options ResolverOptions) remotes.Resolver {
 		// make a copy of the headers to avoid race due to concurrent map write
 		options.Headers = options.Headers.Clone()
 	}
-	if _, ok := options.Headers["User-Agent"]; !ok {
-		options.Headers.Set("User-Agent", "containerd/"+version.Version)
-	}
 
 	resolveHeader := http.Header{}
 	if _, ok := options.Headers["Accept"]; !ok {
@@ -231,7 +230,7 @@ func (r *countingReader) Read(p []byte) (int, error) {
 	return n, err
 }
 
-var _ remotes.Resolver = &dockerResolver{}
+var _ remotes.ResolverWithOptions = &dockerResolver{}
 
 func (r *dockerResolver) Resolve(ctx context.Context, ref string) (string, ocispec.Descriptor, error) {
 	base, err := r.resolveDockerBase(ref)
@@ -284,8 +283,16 @@ func (r *dockerResolver) Resolve(ctx context.Context, ref string) (string, ocisp
 		firstErr         error
 		firstErrPriority int
 	)
+
+	nextHostOrFail := func(i int) string {
+		if i < len(hosts)-1 {
+			return "trying next host"
+		}
+		return "fetch failed"
+	}
+
 	for _, u := range paths {
-		for _, host := range hosts {
+		for i, host := range hosts {
 			ctx := log.WithLogger(ctx, log.G(ctx).WithField("host", host.Host))
 
 			req := base.request(host, http.MethodHead, u...)
@@ -298,7 +305,7 @@ func (r *dockerResolver) Resolve(ctx context.Context, ref string) (string, ocisp
 			}
 
 			log.G(ctx).Debug("resolving")
-			resp, err := req.doWithRetries(ctx, nil)
+			resp, err := req.doWithRetries(ctx, i == len(hosts)-1)
 			if err != nil {
 				if errors.Is(err, ErrInvalidAuthorization) {
 					err = fmt.Errorf("pull access denied, repository does not exist or may require authorization: %w", err)
@@ -307,7 +314,7 @@ func (r *dockerResolver) Resolve(ctx context.Context, ref string) (string, ocisp
 					firstErr = err
 					firstErrPriority = 1
 				}
-				log.G(ctx).WithError(err).Info("trying next host")
+				log.G(ctx).WithError(err).Info(nextHostOrFail(i))
 				continue // try another host
 			}
 			resp.Body.Close() // don't care about body contents.
@@ -318,18 +325,18 @@ func (r *dockerResolver) Resolve(ctx context.Context, ref string) (string, ocisp
 						firstErr = fmt.Errorf("%s: %w", ref, errdefs.ErrNotFound)
 						firstErrPriority = 2
 					}
-					log.G(ctx).Info("trying next host - response was http.StatusNotFound")
+					log.G(ctx).Infof("%s after status: %s", nextHostOrFail(i), resp.Status)
 					continue
 				}
 				if resp.StatusCode > 399 {
 					if firstErrPriority < 3 {
-						firstErr = remoteerrors.NewUnexpectedStatusErr(resp)
+						firstErr = unexpectedResponseErr(resp)
 						firstErrPriority = 3
 					}
-					log.G(ctx).Infof("trying next host - response was %s", resp.Status)
+					log.G(ctx).Infof("%s after status: %s", nextHostOrFail(i), resp.Status)
 					continue // try another host
 				}
-				return "", ocispec.Descriptor{}, remoteerrors.NewUnexpectedStatusErr(resp)
+				return "", ocispec.Descriptor{}, unexpectedResponseErr(resp)
 			}
 			size := resp.ContentLength
 			contentType := getManifestMediaType(resp)
@@ -362,7 +369,7 @@ func (r *dockerResolver) Resolve(ctx context.Context, ref string) (string, ocisp
 					req.header[key] = append(req.header[key], value...)
 				}
 
-				resp, err := req.doWithRetries(ctx, nil)
+				resp, err := req.doWithRetries(ctx, true)
 				if err != nil {
 					return "", ocispec.Descriptor{}, err
 				}
@@ -378,13 +385,8 @@ func (r *dockerResolver) Resolve(ctx context.Context, ref string) (string, ocisp
 					}
 
 					if contentType == images.MediaTypeDockerSchema1Manifest {
-						b, err := schema1.ReadStripSignature(&bodyReader)
-						if err != nil {
-							return err
-						}
-
-						dgst = digest.FromBytes(b)
-						return nil
+						return fmt.Errorf("%w: media type %q is no longer supported since containerd v2.0, please rebuild the image as %q or %q",
+							errdefs.ErrNotImplemented, images.MediaTypeDockerSchema1Manifest, images.MediaTypeDockerSchema2Manifest, ocispec.MediaTypeImageManifest)
 					}
 
 					dgst, err = digest.FromReader(&bodyReader)
@@ -424,6 +426,12 @@ func (r *dockerResolver) Resolve(ctx context.Context, ref string) (string, ocisp
 	return "", ocispec.Descriptor{}, firstErr
 }
 
+func (r *dockerResolver) SetOptions(options ...transfer.ImageResolverOption) {
+	for _, opt := range options {
+		opt(&r.config)
+	}
+}
+
 func (r *dockerResolver) Fetcher(ctx context.Context, ref string) (remotes.Fetcher, error) {
 	base, err := r.resolveDockerBase(ref)
 	if err != nil {
@@ -458,10 +466,25 @@ func (r *dockerResolver) resolveDockerBase(ref string) (*dockerBase, error) {
 }
 
 type dockerBase struct {
-	refspec    reference.Spec
-	repository string
-	hosts      []RegistryHost
-	header     http.Header
+	refspec      reference.Spec
+	repository   string
+	hosts        []RegistryHost
+	header       http.Header
+	performances transfer.ImageResolverPerformanceSettings
+	limiter      *semaphore.Weighted
+}
+
+func (r *dockerBase) Acquire(ctx context.Context, weight int64) error {
+	if r.limiter == nil {
+		return nil
+	}
+	return r.limiter.Acquire(ctx, weight)
+}
+
+func (r *dockerBase) Release(weight int64) {
+	if r.limiter != nil {
+		r.limiter.Release(weight)
+	}
 }
 
 func (r *dockerResolver) base(refspec reference.Spec) (*dockerBase, error) {
@@ -471,10 +494,12 @@ func (r *dockerResolver) base(refspec reference.Spec) (*dockerBase, error) {
 		return nil, err
 	}
 	return &dockerBase{
-		refspec:    refspec,
-		repository: strings.TrimPrefix(refspec.Locator, host+"/"),
-		hosts:      hosts,
-		header:     r.header,
+		refspec:      refspec,
+		repository:   strings.TrimPrefix(refspec.Locator, host+"/"),
+		hosts:        hosts,
+		header:       r.header,
+		performances: r.config.Performances,
+		limiter:      r.config.DownloadLimiter,
 	}, nil
 }
 
@@ -496,6 +521,11 @@ func (r *dockerBase) request(host RegistryHost, method string, ps ...string) *re
 	for key, value := range host.Header {
 		header[key] = append(header[key], value...)
 	}
+
+	if len(header.Get("User-Agent")) == 0 {
+		header.Set("User-Agent", "containerd/"+version.Version)
+	}
+
 	parts := append([]string{"/", host.Path, r.repository}, ps...)
 	p := path.Join(parts...)
 	// Join strips trailing slash, re-add ending "/" if included
@@ -553,6 +583,12 @@ type request struct {
 	size   int64
 }
 
+func (r *request) clone() *request {
+	res := *r
+	res.header = r.header.Clone()
+	return &res
+}
+
 func (r *request) do(ctx context.Context) (*http.Response, error) {
 	u := r.host.Scheme + "://" + r.host.Host + r.path
 	req, err := http.NewRequestWithContext(ctx, r.method, u, nil)
@@ -608,26 +644,91 @@ func (r *request) do(ctx context.Context) (*http.Response, error) {
 	return resp, nil
 }
 
-func (r *request) doWithRetries(ctx context.Context, responses []*http.Response) (*http.Response, error) {
+type doChecks func(r *request, resp *http.Response) error
+
+func withErrorCheck(r *request, resp *http.Response) error {
+	if resp.StatusCode > 299 {
+		if resp.StatusCode == http.StatusNotFound {
+			return fmt.Errorf("content at %v not found: %w", r.String(), errdefs.ErrNotFound)
+		}
+
+		return unexpectedResponseErr(resp)
+	}
+	return nil
+}
+
+var errContentRangeIgnored = errors.New("content range requests ignored")
+
+func withOffsetCheck(offset int64) doChecks {
+	return func(r *request, resp *http.Response) error {
+		if offset == 0 {
+			return nil
+		}
+		if resp.StatusCode == http.StatusPartialContent {
+			return nil
+		}
+		if cr := resp.Header.Get("Content-Range"); cr != "" {
+			if !strings.HasPrefix(cr, fmt.Sprintf("bytes %d-", offset)) {
+				return fmt.Errorf("unhandled content range in response: %v", cr)
+			}
+			return nil
+		}
+
+		// Discard up to offset
+		// Could use buffer pool here but this case should be rare
+		n, err := io.Copy(io.Discard, io.LimitReader(resp.Body, offset))
+		if err != nil {
+			return fmt.Errorf("failed to discard to offset: %w", err)
+		}
+		if n != offset {
+			return errors.New("unable to discard to offset")
+		}
+
+		// content range ignored, we can't do concurrent fetches here.
+		// return an error to be caught
+		return errContentRangeIgnored
+	}
+}
+
+func (r *request) doWithRetries(ctx context.Context, lastHost bool, checks ...doChecks) (resp *http.Response, err error) {
+	resp, err = r.doWithRetriesInner(ctx, nil, lastHost)
+	if err != nil {
+		return nil, err
+	}
+	defer func() {
+		if err != nil && err != errContentRangeIgnored {
+			resp.Body.Close()
+		}
+	}()
+	for _, check := range checks {
+		if err := check(r, resp); err != nil {
+			return resp, err
+		}
+	}
+
+	return resp, nil
+}
+
+func (r *request) doWithRetriesInner(ctx context.Context, responses []*http.Response, lastHost bool) (*http.Response, error) {
 	resp, err := r.do(ctx)
 	if err != nil {
 		return nil, err
 	}
 
 	responses = append(responses, resp)
-	retry, err := r.retryRequest(ctx, responses)
+	retry, err := r.retryRequest(ctx, responses, lastHost)
 	if err != nil {
 		resp.Body.Close()
 		return nil, err
 	}
 	if retry {
 		resp.Body.Close()
-		return r.doWithRetries(ctx, responses)
+		return r.doWithRetriesInner(ctx, responses, lastHost)
 	}
 	return resp, err
 }
 
-func (r *request) retryRequest(ctx context.Context, responses []*http.Response) (bool, error) {
+func (r *request) retryRequest(ctx context.Context, responses []*http.Response, lastHost bool) (bool, error) {
 	if len(responses) > 5 {
 		return false, nil
 	}
@@ -653,9 +754,17 @@ func (r *request) retryRequest(ctx context.Context, responses []*http.Response)
 		}
 	case http.StatusRequestTimeout, http.StatusTooManyRequests:
 		return true, nil
+	case http.StatusServiceUnavailable, http.StatusGatewayTimeout, http.StatusInternalServerError:
+		// Do not retry if the same error was seen in the last request
+		if len(responses) > 1 && responses[len(responses)-2].StatusCode == last.StatusCode {
+			return false, nil
+		}
+		// Only retry if this is the last host that will be attempted
+		if lastHost {
+			return true, nil
+		}
 	}
 
-	// TODO: Handle 50x errors accounting for attempt history
 	return false, nil
 }
 
@@ -663,6 +772,18 @@ func (r *request) String() string {
 	return r.host.Scheme + "://" + r.host.Host + r.path
 }
 
+func (r *request) setMediaType(mediatype string) {
+	if mediatype == "" {
+		r.header.Set("Accept", "*/*")
+	} else {
+		r.header.Set("Accept", strings.Join([]string{mediatype, `*/*`}, ", "))
+	}
+}
+
+func (r *request) setOffset(offset int64) {
+	r.header.Set("Range", fmt.Sprintf("bytes=%d-", offset))
+}
+
 func requestFields(req *http.Request) log.Fields {
 	fields := map[string]interface{}{
 		"request.method": req.Method,
@@ -728,11 +849,16 @@ func NewHTTPFallback(transport http.RoundTripper) http.RoundTripper {
 type httpFallback struct {
 	super http.RoundTripper
 	host  string
+	mu    sync.Mutex
 }
 
 func (f *httpFallback) RoundTrip(r *http.Request) (*http.Response, error) {
+	f.mu.Lock()
+	fallback := f.host == r.URL.Host
+	f.mu.Unlock()
+
 	// only fall back if the same host had previously fell back
-	if f.host != r.URL.Host {
+	if !fallback {
 		resp, err := f.super.RoundTrip(r)
 		if !isTLSError(err) && !isPortError(err, r.URL.Host) {
 			return resp, err
@@ -745,8 +871,12 @@ func (f *httpFallback) RoundTrip(r *http.Request) (*http.Response, error) {
 	plainHTTPRequest := *r
 	plainHTTPRequest.URL = &plainHTTPUrl
 
-	if f.host != r.URL.Host {
-		f.host = r.URL.Host
+	if !fallback {
+		f.mu.Lock()
+		if f.host != r.URL.Host {
+			f.host = r.URL.Host
+		}
+		f.mu.Unlock()
 
 		// update body on the second attempt
 		if r.Body != nil && r.GetBody != nil {
